""" Utils functions"""


def get_sender_id(data):
    """
    :param data: receives facebook object
    :return: User id which wrote a message, type -> str
    """
    sender_id = data['entry'][0]['messaging'][0]['sender']['id']
    return sender_id


def get_user_text(data):
    """
    :param data: receives facebook object
    :return: User text, type -> str
    """
    text = data['entry'][0]['messaging'][0]['message']['text']
    return text



import requests as r

from .constants import API_URL
from .exc import BotChuckyInvalidToken, BotChuckyTokenError
from .helpers import FacebookData, WeatherData


class BotChucky:
    def __init__(self, token, open_weather_token=None):
        """
        :param token: Facebook Token, required
        :param open_weather_token: not required
        :param headers: Set default headers for the graph API, default
        :param fb: Instace of FacebookData class, default
        :param weather: Instace of WeatherData class, default
        """
        self.token = token
        self.open_weather_token = open_weather_token
        self.params = {'access_token': self.token}
        self.headers = {'Content-Type': 'application/json'}
        self.fb = FacebookData(self.token)
        self.weather = WeatherData(open_weather_token)

    def send_message(self, _id, text):
        """
        :param _id: User facebook id, type -> str
        :param text: some text, type -> str
        """
        data = {
            'recipient': {'id': _id},
            'message': {'text': text}
        }
        message = r.post(API_URL, params=self.params,
                         headers=self.headers, json=data)
        if message.status_code is not 200:
            return message.text

    def send_weather_message(self, _id, city_name):
        """
        :param _id: User facebook id, type -> str
        :param city_name: Find weather by city name
        :return send_message function,
         send message to a user, with current weather
        """
        if self.open_weather_token is None:
            raise BotChuckyTokenError

        weather_info = self.weather.get_current_weather(city_name)
        if weather_info['cod'] == 401:
            error = weather_info['message']
            raise BotChuckyInvalidToken(error)

        if weather_info['cod'] == '404':
            msg = f'Sorry I can\'t find information ' \
                  f'about weather in {city_name}, ' \
                  f'please check your name of city'
            return self.send_message(_id, msg)

        description = weather_info['weather'][0]['description']
        msg = f'Current weather in {city_name} is: {description}'
        return self.send_message(_id, msg)



""" Helper classes """

import facebook
import requests as r


class FacebookData:
    def __init__(self, token):
        """
        :param token: Facebook Page token
        :param _api: Instance of the GraphAPI object
        """
        self.token = token
        self._api = facebook.GraphAPI(self.token)

    def get_user_name(self, _id):
        """
        :param _id: find user object by _id
        :return: first name of user, type -> str
        """
        if not isinstance(_id, str):
            raise ValueError('id must be a str')
        user = self._api.get_object(_id)
        return user['first_name'] if user else None


class WeatherData:
    """
    Class which collect weather data
    """
    def __init__(self, api_token):
        """
        :param api_token: Open Weather TOKEN
        """
        self.token = api_token

    def get_current_weather(self, city_name):
        """
        :param city_name: Open weather API, find by city name
        :return dictionary object with information

        for example:

        {'weather': [{'id': 800, 'main': 'Clear', 'description': 'clear sky'}]}
        """
        api_url = f'http://api.openweathermap.org' \
                  f'/data/2.5/weather?q={city_name}&APPID={self.token}'

        info = r.get(api_url).json()
        return info



from setuptools import setup

import bot_chucky

version = bot_chucky.__version__

setup_kwargs = {
    'name': 'bot_chucky',
    'version': version,
    'url': 'https://github.com/MichaelYusko/Bot-Chucky',
    'license': 'MIT',
    'author': 'Frozen Monkey',
    'author_email': 'freshjelly12@yahoo.com',
    'description': 'Python bot which able to work with messenger of facebook',
    'packages': ['bot_chucky'],
    'classifiers': [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: MIT License'
    ],
 }

requirements = ['requests==2.17.3', 'facebook-sdk==2.0.0']
setup_kwargs['install_requires'] = requirements

setup(**setup_kwargs)

print(u"\n\n\t\t    "
      "BotChucky version {} installation succeeded.\n".format(version))



""" Exception classes """


class BotChuckyTokenError(Exception):
    def __str__(self):
        return 'Seems like you missing add' \
               ' \'Open Weather\' token to the ChuckyBot class'


class BotChuckyInvalidToken(Exception):
    pass
